@@ -20,34 +20,21 @@
 
 #include <string.h>
 
-#if HAVE_GIO
-#include <gio/gio.h>
-#else
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#endif
+#include <errno.h>
 
 #include <glib-object.h>
-
-#include "gegl-types-internal.h"
+#include <glib/gstdio.h>
 
 #include "gegl.h"
 #include "gegl-buffer-types.h"
-#include "gegl-buffer.h"
 #include "gegl-buffer-private.h"
 #include "gegl-debug.h"
 #include "gegl-tile-storage.h"
-#include "gegl-tile-backend.h"
-#include "gegl-tile-handler.h"
 #include "gegl-tile.h"
-#include "gegl-tile-handler-cache.h"
-#include "gegl-tile-handler-log.h"
-#include "gegl-tile-handler-empty.h"
-#include "gegl-types-internal.h"
-#include "gegl-utils.h"
-#include "gegl-buffer-save.h"
 #include "gegl-buffer-index.h"
 
 typedef struct
@@ -55,12 +42,7 @@ typedef struct
   GeglBufferHeader header;
   GList           *tiles;
   gchar           *path;
-#if HAVE_GIO
-  GFile           *file;
-  GOutputStream   *o;
-#else
-  int             o;
-#endif
+  gint             o;
 
   gint             tile_size;
   gint             offset;
@@ -106,13 +88,9 @@ static gsize write_block (SaveInfo        *info,
        if (block == NULL)
          info->in_holding->next = 0;
 
-#if HAVE_GIO
-       ret = g_output_stream_write (info->o, info->in_holding, info->in_holding->length, NULL, NULL);
-#else
        ret = write (info->o, info->in_holding, info->in_holding->length);
-	   if (ret == -1) 
+	   if (ret == -1)
          ret = 0;
-#endif
        info->offset += ret;
        g_assert (allocated_pos == info->offset);
      }
@@ -128,17 +106,9 @@ save_info_destroy (SaveInfo *info)
 {
   if (!info)
     return;
-  if (info->path)
-    g_free (info->path);
-#if HAVE_GIO
-  if (info->o)
-    g_object_unref (info->o);
-  if (info->file)
-    g_object_unref (info->file);
-#else
+  g_free (info->path);
   if (info->o != -1)
     close (info->o);
-#endif
   if (info->tiles != NULL)
     {
       GList *iter;
@@ -205,7 +175,7 @@ gegl_buffer_header_init (GeglBufferHeader *header,
   {
     gchar buf[64] = { 0, };
 
-    g_snprintf (buf, 64, "%s%c\n%i×%i %ibpp\n%ix%i\n\n\n\n\n\n\n\n\n", 
+    g_snprintf (buf, 64, "%s%c\n%i×%i %ibpp\n%ix%i\n\n\n\n\n\n\n\n\n",
           babl_get_name (format), 0,
           header->tile_width,
           header->tile_height,
@@ -223,7 +193,7 @@ gegl_buffer_save (GeglBuffer          *buffer,
 {
   SaveInfo *info = g_slice_new0 (SaveInfo);
 
-  glong prediction = 0; 
+  glong prediction = 0;
   gint bpp;
   gint tile_width;
   gint tile_height;
@@ -243,12 +213,16 @@ gegl_buffer_save (GeglBuffer          *buffer,
    */
 
   info->path = g_strdup (path);
-#if HAVE_GIO
-  info->file = g_file_new_for_commandline_arg (info->path);
-  info->o    = G_OUTPUT_STREAM (g_file_replace (info->file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, NULL));
+
+#ifndef G_OS_WIN32
+  info->o    = g_open (info->path, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
 #else
-  info->o    = open (info->path, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
+  info->o    = g_open (info->path, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR);
 #endif
+
+
+  if (info->o == -1)
+    g_warning ("%s: Could not open '%s': %s", G_STRFUNC, info->path, g_strerror(errno));
   tile_width  = buffer->tile_storage->tile_width;
   tile_height = buffer->tile_storage->tile_height;
   g_object_get (buffer, "px-size", &bpp, NULL);
@@ -273,11 +247,10 @@ gegl_buffer_save (GeglBuffer          *buffer,
   {
     gint z;
     gint factor = 1;
-    int  bufy = roi->y;
 
     for (z = 0; z < 1; z++)
       {
-        bufy = roi->y;
+        gint bufy = roi->y;
         while (bufy < roi->y + roi->height)
           {
             gint tiledy  = roi->y + bufy;
@@ -334,15 +307,11 @@ gegl_buffer_save (GeglBuffer          *buffer,
   }
 
   /* save the header */
-#if HAVE_GIO
-  info->offset += g_output_stream_write (info->o, &info->header, sizeof (GeglBufferHeader), NULL, NULL);
-#else
   {
     ssize_t ret = write (info->o, &info->header, sizeof (GeglBufferHeader));
 	if (ret != -1)
       info->offset += ret;
   }
-#endif
   g_assert (info->offset == info->header.next);
 
   /* save the index */
@@ -381,15 +350,11 @@ gegl_buffer_save (GeglBuffer          *buffer,
         g_assert (data);
 
         g_assert (info->offset == entry->offset);
-#if HAVE_GIO
-        info->offset += g_output_stream_write (info->o, data, info->tile_size, NULL, NULL);
-#else
-        { 
+        {
           ssize_t ret = write (info->o, data, info->tile_size);
 	      if (ret != -1)
             info->offset += ret;
         }
-#endif
         gegl_tile_unref (tile);
         i++;
       }
