@@ -19,8 +19,6 @@
 #ifndef __GEGL_BUFFER_TYPES_H__
 #define __GEGL_BUFFER_TYPES_H__
 
-typedef struct _GeglTileClass             GeglTileClass;
-
 #include "gegl-types.h"
 #include "gegl-buffer-backend.h"
 
@@ -32,39 +30,25 @@ typedef struct _GeglTileClass             GeglTileClass;
 
 struct _GeglTileBackendPrivate
 {
-  gint       tile_width;
-  gint       tile_height;
-  Babl      *format;    /* defaults to the babl format "R'G'B'A u8" */
-  gint       px_size;   /* size of a single pixel in bytes */
-  gint       tile_size; /* size of an entire tile in bytes */
+  gint        tile_width;
+  gint        tile_height;
+  const Babl *format;    /* defaults to the babl format "R'G'B'A u8" */
+  gint        px_size;   /* size of a single pixel in bytes */
+  gint        tile_size; /* size of an entire tile in bytes */
+
+  gboolean    flush_on_destroy;
 
   GeglRectangle extent;
 
-  gpointer   header;
-  gpointer   storage;
-  gboolean   shared;
+  gpointer    storage;
+  gboolean    shared;
 };
 
-
-
-typedef struct _GeglTileHandler           GeglTileHandler;
-typedef struct _GeglTileHandlerClass      GeglTileHandlerClass;
-
 typedef struct _GeglTileHandlerChain      GeglTileHandlerChain;
-typedef struct _GeglTileHandlerChainClass GeglTileHandlerChainClass;
 
 typedef struct _GeglTileStorage           GeglTileStorage;
-typedef struct _GeglTileStorageClass      GeglTileStorageClass;
-
-#ifndef __GEGL_BUFFER_H__
-typedef struct _GeglBuffer                GeglBuffer;
-typedef struct _GeglSampler               GeglSampler;
-#endif
-
-typedef struct _GeglBufferClass           GeglBufferClass;
 
 typedef struct _GeglCache                 GeglCache;
-typedef struct _GeglCacheClass            GeglCacheClass;
 
 typedef struct _GeglRegion                GeglRegion;
 
