@@ -48,7 +48,7 @@ enum
   PROP_APPLICATION_LICENSE
 };
 
-gint _gegl_threads = 1; 
+gint _gegl_threads = 1;
 
 static void
 gegl_config_get_property (GObject    *gobject,
@@ -225,11 +225,13 @@ gegl_config_class_init (GeglConfigClass *klass)
                                                         G_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT));
 
+  _gegl_threads = g_get_num_processors ();
   g_object_class_install_property (gobject_class, PROP_THREADS,
                                    g_param_spec_int ("threads",
                                                      "Number of threads",
                                                      "Number of concurrent evaluation threads",
-                                                     0, GEGL_MAX_THREADS, 1,
+                                                     0, GEGL_MAX_THREADS,
+                                                     _gegl_threads,
                                                      G_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT));
 
@@ -237,7 +239,7 @@ gegl_config_class_init (GeglConfigClass *klass)
                                    g_param_spec_boolean ("use-opencl",
                                                          "Use OpenCL",
                                                          "Try to use OpenCL",
-                                                         TRUE,
+                                                         FALSE,
                                                          G_PARAM_READWRITE |
                                                          G_PARAM_CONSTRUCT));
 
