@@ -146,8 +146,7 @@ gegl_init_swap_dir (void)
       ! g_file_test (swapdir, G_FILE_TEST_IS_DIR) &&
       g_mkdir_with_parents (swapdir, S_IRUSR | S_IWUSR | S_IXUSR) != 0)
     {
-      g_free (swapdir);
-      swapdir = NULL;
+      g_clear_pointer (&swapdir, g_free);
     }
 
   g_object_set (config, "swap", swapdir, NULL);
@@ -170,14 +169,21 @@ gegl_swap_dir (void)
   return swap_dir;
 }
 
+static void load_module_path(gchar *path, GeglModuleDB *db);
+
 static void
 gegl_config_application_license_notify (GObject    *gobject,
                                         GParamSpec *pspec,
                                         gpointer    user_data)
 {
   GeglConfig *cfg = GEGL_CONFIG (gobject);
+  GSList *paths = gegl_get_default_module_paths ();
 
   gegl_operations_set_licenses_from_string (cfg->application_license);
+
+  /* causes load of .so's that might have been skipped due to filename */
+  g_slist_foreach(paths, (GFunc)load_module_path, module_db);
+  g_slist_free_full (paths, g_free);
 }
 
 
@@ -468,11 +474,7 @@ gegl_exit (void)
 
   gegl_temp_buffer_free ();
 
-  if (module_db != NULL)
-    {
-      g_object_unref (module_db);
-      module_db = NULL;
-    }
+  g_clear_object (&module_db);
 
   babl_exit ();
 
@@ -536,8 +538,7 @@ gegl_exit (void)
 
       g_pattern_spec_free (pattern);
     }
-  g_object_unref (config);
-  config = NULL;
+  g_clear_object (&config);
   global_time = 0;
 }
 
