@@ -35,6 +35,7 @@
 
 #include "opencl/gegl-cl.h"
 #include "gegl-buffer-cl-iterator.h"
+#include "gegl-buffer-cl-cache.h"
 
 typedef struct ThreadData
 {
@@ -289,18 +290,33 @@ gegl_operation_point_filter_process (GeglOperation       *operation,
         GThreadPool *pool = thread_pool ();
         gint pending;
         int j;
-        GeglRectangle sub_result;
+
+        if (result->width > result->height)
+          for (j = 0; j < threads; j++)
+          {
+            GeglRectangle rect = *result;
+            rect.width /= threads;
+            rect.x += rect.width * j;
+
+            if (j == threads-1)
+              rect.width = (result->width + result->x) - rect.x;
+
+            thread_data[j].result = rect;
+          }
+        else
+          for (j = 0; j < threads; j++)
+          {
+            GeglRectangle rect = *result;
+            rect = *result;
+            rect.height /= threads;
+            rect.y += rect.height * j;
+            if (j == threads-1)
+              rect.height = (result->height + result->y) - rect.y;
+            thread_data[j].result = rect;
+          }
 
         for (j = 0; j < threads; j++)
         {
-          sub_result = *result;
-          sub_result.height /= threads;
-          sub_result.y += sub_result.height * j;
-          if (j == threads-1)
-          {
-            sub_result.height = (result->height + result->y) - sub_result.y;
-          }
-
           thread_data[j].klass = point_filter_class;
           thread_data[j].operation = operation;
           thread_data[j].input = input;
@@ -309,11 +325,13 @@ gegl_operation_point_filter_process (GeglOperation       *operation,
           thread_data[j].level = level;
           thread_data[j].input_format = in_format;
           thread_data[j].output_format = out_format;
-          thread_data[j].result = sub_result;
         }
 
         pending = threads;
 
+        if (gegl_cl_is_accelerated () && input)
+          gegl_buffer_cl_cache_flush (input, result);
+
         for (gint j = 1; j < threads; j++)
           g_thread_pool_push (pool, &thread_data[j], NULL);
         thread_process (&thread_data[0], NULL);
