@@ -173,7 +173,7 @@ gegl_operation_process (GeglOperation        *operation,
 }
 
 /* Calls an extending class' get_bound_box method if defined otherwise
- * just returns a zero-initiliased bouding box
+ * just returns a zero-initialised bounding box
  */
 GeglRectangle
 gegl_operation_get_bounding_box (GeglOperation *self)
@@ -349,12 +349,21 @@ GeglNode *
 gegl_operation_get_source_node (GeglOperation *operation,
                                 const gchar   *input_pad_name)
 {
+  GeglNode *node;
   GeglPad *pad;
 
-  g_assert (operation &&
-            operation->node &&
-            input_pad_name);
-  pad = gegl_node_get_pad (operation->node, input_pad_name);
+  g_return_val_if_fail (GEGL_IS_OPERATION (operation), NULL);
+  g_return_val_if_fail (GEGL_IS_NODE (operation->node), NULL);
+  g_return_val_if_fail (input_pad_name != NULL, NULL);
+
+  node = operation->node;
+  if (node->is_graph)
+    {
+      node = gegl_node_get_input_proxy (node, input_pad_name);
+      input_pad_name = "input";
+    }
+
+  pad = gegl_node_get_pad (node, input_pad_name);
 
   if (!pad)
     return NULL;
