@@ -926,7 +926,7 @@ gegl_expcombine_get_exposures (GeglOperation        *operation,
         }
 
       /* Add exposure to list */
-      buffer  = gegl_operation_context_get_source (context,
+      buffer = (GeglBuffer*)gegl_operation_context_dup_object (context,
                                                    gegl_pad_get_name (pad));
       if (!buffer)
           continue;
@@ -954,6 +954,7 @@ gegl_expcombine_get_exposures (GeglOperation        *operation,
         }
 
       e->pixels[PIXELS_ACTIVE] = e->pixels[PIXELS_FULL];
+      g_object_unref (buffer);
 
       /* Read the exposure time: relate APEX brightness value only as a
        * function of exposure time that is assume aperture = 1 and
