@@ -90,6 +90,8 @@ gegl_op_class_init (GeglOpClass *klass)
   gegl_operation_class_set_keys (operation_class,
     "name"       , "gegl:svg-luminancetoalpha",
     "categories" , "compositors:svgfilter",
+    "title"      , _("SVG Luminance to Alpha"),
+    "reference-hash", "6f941960c9e8ea9210fad5483f55cdfc",
     "description",
        _("SVG color matrix operation svg_luminancetoalpha"),
         NULL);
