@@ -168,7 +168,9 @@ op_transform_class_init (OpTransformClass *klass)
   op_class->process                   = gegl_transform_process;
   op_class->prepare                   = gegl_transform_prepare;
   op_class->no_cache                  = TRUE;
-  op_class->threaded                  = TRUE;
+  op_class->threaded                  = FALSE; // XXX : at the moment threading
+                                               // produces right result but
+                                               // slows down most transform - and significantly slows down scale
 
   klass->create_matrix = NULL;
 
@@ -1145,7 +1147,7 @@ gegl_transform_process (GeglOperation        *operation,
       gegl_matrix3_is_identity (&matrix))
     {
       /* passing straight through (like gegl:nop) */
-      input  = gegl_operation_context_get_source (context, "input");
+      input  = (GeglBuffer*)gegl_operation_context_dup_object (context, "input");
       if (!input)
         {
           g_warning ("transform received NULL input");
@@ -1167,7 +1169,7 @@ gegl_transform_process (GeglOperation        *operation,
        * TODO: Should not be taken by non-interpolatory samplers (the
        * current cubic, for example).
        */
-      input  = gegl_operation_context_get_source (context, "input");
+      input  = (GeglBuffer*) gegl_operation_context_dup_object (context, "input");
       output =
         g_object_new (GEGL_TYPE_BUFFER,
                       "source", input,
@@ -1199,7 +1201,7 @@ gegl_transform_process (GeglOperation        *operation,
       /*
        * For all other cases, do a proper resampling
        */
-      input  = gegl_operation_context_get_source (context, "input");
+      input  = (GeglBuffer*) gegl_operation_context_dup_object (context, "input");
       output = gegl_operation_context_get_target (context, "output");
 
       /* flush opencl caches, to avoid racy flushing
